@@ -2,6 +2,14 @@ import os
 import json
 import click
 
+class DelimitedStr(click.Option):
+
+    def type_cast_value(self, ctx, value):
+        try:
+            return [i.strip() for i in value.split(",")]
+        except:
+            raise click.BadParameter(value)
+
 @click.command(name='run_pulls')
 @click.option('-i', '--input_file', required=True, 
               help='Path to the input workspace file')
