@@ -5,6 +5,8 @@ import json
 
 import numpy as np
 
+import ROOT
+
 from quickstats import semistaticmethod
 from quickstats.components import AnalysisObject
 from quickstats.utils.root_utils import load_macro
@@ -13,8 +15,6 @@ from quickstats.utils.io import Verbosity
 from quickstats.maths.numerics import pretty_value
 from quickstats.components.basics import WSArgument
 
-import ROOT
-
 class AsymptoticCLs(AnalysisObject):
     
     _SIGMA_LABEL_ = {
@@ -97,8 +97,8 @@ class AsymptoticCLs(AnalysisObject):
             if not asimov_data_0:
                 raise RuntimeError(f'failed to load dataset \"{asimov_data_name}\"')
             self.asimov_data_0 = asimov_data_0
-            glob_snap_name = self.get_conditional_snapsot_name(0, "global_observables")
-            nuis_snap_name = self.get_conditional_snapsot_name(0, "nuisance_parameters")
+            glob_snap_name = self.get_conditional_snapsot_name(0, "global_observable")
+            nuis_snap_name = self.get_conditional_snapsot_name(0, "nuisance_parameter")
             if (not self.model.workspace.loadSnapshot(glob_snap_name)) or \
             (not self.model.workspace.loadSnapshot(nuis_snap_name)):
                 raise RuntimeError("when using user-defined background-only Asimov data, "
