@@ -105,15 +105,17 @@ class ExtendedModel(AbstractObject):
     @property
     def observables(self):
         return self._observables
-    
     @property
     def category(self):
         return self._category
-    
+    @property
+    def floating_auxiliary_variables(self):
+        return self._floating_auxiliary_variables
+    """
     @property
     def data_cat(self):
         return self._data_cat
-    
+    """
     @property
     def initial_snapshots(self):
         return self._initial_snapshots
@@ -305,12 +307,13 @@ class ExtendedModel(AbstractObject):
             category = pdf.indexCat()
         else:
             category = None
-        
+        """
         # split dataset according to categories
         if category is not None:
             data_cat = data.split(category, True)
         else:
             data_cat = None
+        """
         
         self._file                = file
         self._workspace           = ws
@@ -322,7 +325,8 @@ class ExtendedModel(AbstractObject):
         self._pois                = pois
         self._observables         = observables
         self._category            = category
-        self._data_cat            = data_cat
+        #self._data_cat            = data_cat
+        self._floating_auxiliary_variables = None
                           
         # Load snapshots
         self.load_snapshots(self.initial_snapshots)
@@ -969,7 +973,8 @@ class ExtendedModel(AbstractObject):
             poi = self.get_poi(poi_name)
         
         # take snapshot of initial states of all variables
-        ws.saveSnapshot(nom_vars_name, self.workspace.allVars())
+        mutable_vars = self.get_variables(WSArgument.MUTABLE)
+        ws.saveSnapshot(nom_vars_name, mutable_vars)
         if not ws.getSnapshot(nom_glob_name):
             self.stdout.info('INFO: Saving snapshot "{}"'.format(nom_glob_name))
             ws.saveSnapshot(nom_glob_name, all_globs)
@@ -1060,10 +1065,10 @@ class ExtendedModel(AbstractObject):
             
         if asimov_snapshot is not None:
             self.stdout.info('INFO: Saving snapshot "{}"'.format(asimov_snapshot))
-            ws.saveSnapshot(asimov_snapshot, self.workspace.allVars())
+            ws.saveSnapshot(asimov_snapshot, mutable_vars)
     
         if restore_states == 0:
-            # load back a snapshot of all varaible's initial states
+            # load back a snapshot of all variable's initial states
             ws.loadSnapshot(nom_vars_name)
         elif restore_states == 1:
             # load back a snapshot of the initial global observable states
@@ -1435,25 +1440,30 @@ class ExtendedModel(AbstractObject):
             pdf.close()
         return None
     
+    def _load_floating_auxiliary_variables(self):
+        aux_vars = self.get_variables(WSArgument.AUXILIARY)
+        floating_aux_vars = RooArgSet.from_list([v for v in aux_vars if not v.isConstant()])
+        self._floating_auxiliary_variables = floating_aux_vars
+    
     def get_variables(self, variable_type:Union[str, WSArgument], sort:bool=True):
         resolved_vtype = WSArgument.parse(variable_type)
         if not resolved_vtype.is_variable:
             raise ValueError(f"the collection \"{variable_type}\" does not contain members of the type \"RooRealVar\"")
         if resolved_vtype == WSArgument.VARIABLE:
-            variables = self.workspace.allVars()
+            variables = ROOT.RooArgSet(self.workspace.allVars())
         elif resolved_vtype == WSArgument.OBSERVABLE:
             if isinstance(self.pdf, ROOT.RooSimultaneous):
                 variables = self.observables.Clone()
                 cat = self.pdf.indexCat()
                 variables.remove(cat)
             else:
-                variables = self.observables
+                variables = ROOT.RooArgSet(self.observables)
         elif resolved_vtype == WSArgument.POI:
-            variables = self.pois
+            variables = ROOT.RooArgSet(self.pois)
         elif resolved_vtype == WSArgument.GLOBAL_OBSERVABLE:
-            variables = self.global_observables
+            variables = ROOT.RooArgSet(self.global_observables)
         elif resolved_vtype == WSArgument.NUISANCE_PARAMETER:
-            variables = self.nuisance_parameters
+            variables = ROOT.RooArgSet(self.nuisance_parameters)
         elif resolved_vtype == WSArgument.CONSTRAINED_NUISANCE_PARAMETER:
             variables = RooArgSet.from_list(self.get_constrained_nuisance_parameters())
         elif resolved_vtype == WSArgument.UNCONSTRAINED_NUISANCE_PARAMETER:
@@ -1476,6 +1486,11 @@ class ExtendedModel(AbstractObject):
             variables.add(self.nuisance_parameters)
             variables.add(self.global_observables)
             variables.add(self.pois)
+        elif resolved_vtype == WSArgument.MUTABLE:
+            variables = self.get_variables(WSArgument.CORE)
+            if self.floating_auxiliary_variables is None:
+                self._load_floating_auxiliary_variables()
+            variables.add(self.floating_auxiliary_variables)
         else:
             raise ValueError(f"unknown variable type \"{variable_type}\"")
         if sort:
