@@ -287,10 +287,10 @@ class NuisanceParameterPull(object):
         model = ExtendedModel(input_file, workspace, model_config, data_name=data, verbosity="WARNING")
         # include constrained NPs only
         if constrained_only:
-            nuis_list = [np.GetName() for np in model.get_variables("constrained_nuisance_parameters")]
+            nuis_list = [np.GetName() for np in model.get_variables("constrained_nuisance_parameter")]
         # include all NPs
         else:
-            nuis_list = [np.GetName() for np in model.get_variables("nuisance_parameters")]
+            nuis_list = [np.GetName() for np in model.get_variables("nuisance_parameter")]
         include_patterns = parameter.split(',')
         if parameter:
             nuis_to_include = []
