@@ -16,7 +16,7 @@ from quickstats.utils.root_utils import declare_expression, close_all_root_files
 class RooProcessor(AbstractObject):
     def __init__(self, config_path:Optional[str]=None,
                  multithread:bool=True, cache:bool=False,
-                 verbosity:Optional[Union[int, str]]="INFO"):
+                 use_template:bool=False, verbosity:Optional[Union[int, str]]="INFO"):
         super().__init__(verbosity=verbosity)
         self.cache = cache
         self.action_list = []
@@ -25,6 +25,7 @@ class RooProcessor(AbstractObject):
         self.global_variables = {}
         self.external_variables = {}
         self.treename = None
+        self.use_template = use_template
         
         self.load_buildin_functions()
         
