@@ -8,7 +8,6 @@ import time
 import uuid
 
 import numpy as np
-import pandas as pd
 
 import ROOT
 
@@ -30,6 +29,20 @@ class SignalModelling(AbstractObject):
         'hesse': True,
         'sumw2': True
     }
+    
+    _DEFAULT_PLOT_OPTION_ = {
+        'bin_range': None,
+        'n_bins_data': None,
+        'n_bins_pdf': 1000,
+        'plot_comparison': True,
+        'comparison_mode': 'difference',
+        'show_params': True,
+        'show_stats': True,
+        'styles': None,
+        "xlabel": None,
+        "ylabel": "Events / ({bin_width:.2f})",
+        "annotations": None
+    }
 
     _EXTERNAL_PDF_ = ['RooTwoSidedCBShape']
     
@@ -40,6 +53,10 @@ class SignalModelling(AbstractObject):
         'Bukin': 'RooBukinPdf'
     }
     
+    @property
+    def plot_options(self):
+        return self._plot_options
+    
     @property
     def fit_options(self):
         return self._fit_options
@@ -56,7 +73,8 @@ class SignalModelling(AbstractObject):
                  functional_form:Union[str, Callable],
                  param_templates:Optional[Callable]=None,
                  weight:str="weight", n_bins:Optional[int]=None,
-                 fit_options:Optional[Dict]=None, verbosity:str="INFO"):
+                 fit_options:Optional[Dict]=None, plot_options:Optional[Dict]=None,
+                 verbosity:str="INFO"):
         super().__init__(verbosity=verbosity)
         _fit_options = {
             'observable': observable,
@@ -66,6 +84,7 @@ class SignalModelling(AbstractObject):
             'n_bins': n_bins
         }
         self._fit_options = combine_dict(self._DEFAULT_FIT_OPTION_)
+        self._plot_options = combine_dict(self._DEFAULT_PLOT_OPTION_, plot_options)
         self.update_fit_options(_fit_options)
         self.update_fit_options(fit_options)
         self._param_templates = param_templates
@@ -87,6 +106,9 @@ class SignalModelling(AbstractObject):
     def update_fit_options(self, options:Optional[Dict]=None):
         self._fit_options = combine_dict(self._fit_options, options)
         
+    def update_plot_options(self, options:Optional[Dict]=None):
+        self._plot_options = combine_dict(self._plot_options, options)        
+        
     def set_param_templates(self, param_templates:Callable):
         self._param_templates = param_templates
         
@@ -228,6 +250,63 @@ class SignalModelling(AbstractObject):
                     observable.setRange("fitRange", new_vmin, new_vmax)
         return fit_result
     
+    @staticmethod
+    def _set_pdf_param_values(pdf:ROOT.RooAbsPdf, observable:ROOT.RooRealVar, param_values:Dict):
+        params = pdf.getParameters(observable)
+        for param in params:
+            param_name = param.GetName()
+            if param_name not in param_values:
+                raise RuntimeError(f"missing value for the parameter: {param_name}")
+            param_value = param_values[param_name]
+            param.setVal(param_value)
+    
+    @staticmethod
+    def export_hist_data(data:ROOT.RooDataSet, pdf:ROOT.RooAbsPdf, observable:ROOT.RooRealVar,
+                         bin_range:Optional[List]=None, n_bins_data:Optional[int]=None,
+                         n_bins_pdf:Optional[int]=1000, bin_error:bool=False):
+        if bin_range is None:
+            bin_range = observable.getRange()
+            bin_range = [bin_range.first, bin_range.second]
+            
+        if n_bins_data is None:
+            n_bins_data = observable.getBins()
+        
+        if n_bins_pdf is None:
+            n_bins_pdf = n_bins_data
+        binning_data = ROOT.RooFit.Binning(n_bins_data, bin_range[0], bin_range[1])
+        binning_pdf = ROOT.RooFit.Binning(n_bins_pdf, bin_range[0], bin_range[1])
+        from quickstats.interface.root import RooAbsData
+        h_data = RooAbsData.create_histogram(data, uuid.uuid4().hex, observable, binning_data)
+        h_pdf = RooAbsData.create_histogram(pdf, uuid.uuid4().hex, observable, binning_pdf)
+        h_pdf_data_binning = RooAbsData.create_histogram(pdf, uuid.uuid4().hex, observable,
+                                                         binning_data)
+        from quickstats.interface.root import TH1
+        if bin_error:
+            bin_errors = TH1.GetPoissonError(h_data.bin_content)
+        else:
+            size = len(h_data.bin_content)
+            bin_errors = {"lo": np.zeros(size), "hi": np.zeros(size)}
+        norm_data  = h_data.bin_content.sum()
+        norm_pdf1  = h_pdf.bin_content.sum()
+        norm_pdf2  = h_pdf_data_binning.bin_content.sum()
+        hist_data = {
+            'data': {
+                'x': h_data.bin_center,
+                'y': h_data.bin_content,
+                'yerrlo': bin_errors['lo'],
+                'yerrhi': bin_errors['hi']
+            },
+            'pdf': {
+                'x': h_pdf.bin_center,
+                'y': h_pdf.bin_content * norm_data / norm_pdf1 * (n_bins_pdf / n_bins_data)
+            },
+            'pdf_data_binning': {
+                'x': h_pdf_data_binning.bin_center,
+                'y': h_pdf_data_binning.bin_content * norm_data / norm_pdf2
+            }
+        }
+        return hist_data    
+    
     @staticmethod
     def get_fit_stats(model:"ROOT.RooAbsPdf", data:"ROOT.RooAbsData", observable:"ROOT.RooRealVar",
                       n_float_params:int=0):
@@ -262,7 +341,8 @@ class SignalModelling(AbstractObject):
         return param_summary
 
     def _run(self, fname:str, tree_name:str, model_class:Callable, param_templates:Callable,
-             fit_options:Dict, save_as:Optional[str]=None, simplified_output:bool=False):
+             fit_options:Dict, plot_options:Optional[Dict]=None, save_summary_as:Optional[str]=None, 
+             save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None):
         t1 = time.time()
         c = ROOT.TCanvas(uuid.uuid4().hex)
         f = ROOT.TFile(fname)
@@ -314,70 +394,220 @@ class SignalModelling(AbstractObject):
                    "time": time_taken}
         if not isinstance(summary['fit_options']['functional_form'], str):
             summary['fit_options']['functional_form'] = type(summary['fit_options']['functional_form']).__name__
-        if simplified_output:
-            summary = {}
-            for param in param_summary:
-                summary[param] = param_summary[param]['value']
-        if save_as is not None:
-            json.dump(summary, open(save_as, "w"), indent=2)
+            
+        param_data = self._get_simplified_param_data(param_summary)
+            
+        if save_plot_as is not None:
+            if plot_options is None:
+                plot_options = combine_dict(self._DEFAULT_PLOT_OPTION_)
+            hist_data = self.export_hist_data(data, model_pdf, observable, 
+                                              bin_range=plot_options['bin_range'],
+                                              n_bins_data=plot_options['n_bins_data'],
+                                              n_bins_pdf=plot_options['n_bins_pdf'])
+            kwargs = {
+                "xlabel": plot_options['xlabel'],
+                "ylabel": plot_options['ylabel'],
+                "plot_comparison" : plot_options['plot_comparison'],
+                "comparison_mode" : plot_options['comparison_mode'],
+                "styles" : plot_options['styles'],
+                "annotations": plot_options['annotations']
+            }
+            if plot_options['show_params']:
+                kwargs["param_data"] = param_data
+            if plot_options["show_stats"]:
+                kwargs["stat_data"] = {"chi2/ndf": summary["stats"]["chi2/ndf"]}
+            self.create_plot(hist_data, outname=save_plot_as, **kwargs)
         # free memory
         c.Close()
         ROOT.gSystem.ProcessEvents()
-        return summary
+ 
+        if save_param_as is not None:
+            with open(save_param_as, "w") as out:
+                json.dump(param_data, out, indent=2)
+        if save_summary_as is not None:
+            with open(save_summary_as, "w") as out:
+                json.dump(summary, out, indent=2)
 
-    def run(self, fname:str, tree_name:str, save_as:Optional[str]=None, 
-            simplified_output:bool=False):
-        self._run(fname, tree_name, model_class=self.model_class, 
-                  param_templates=self.param_templates,
-                  fit_options=self.fit_options, save_as=save_as,
-                  simplified_output=simplified_output)
+        return summary
+    
+    @staticmethod
+    def _get_simplified_param_data(param_data:Dict):
+        simplified_param_data = {}
+        for param in param_data:
+            simplified_param_data[param] = param_data[param]['value']
+        return simplified_param_data
+       
+    @staticmethod
+    def export_hist_data(data:ROOT.RooDataSet, pdf:ROOT.RooAbsPdf, observable:ROOT.RooRealVar,
+                         bin_range:Optional[List]=None, n_bins_data:Optional[int]=None,
+                         n_bins_pdf:Optional[int]=1000, bin_error:bool=False):
+        if bin_range is None:
+            bin_range = observable.getRange()
+            bin_range = [bin_range.first, bin_range.second]
+            
+        if n_bins_data is None:
+            n_bins_data = observable.getBins()
         
-    def batch_run(self, fnames:List[str], tree_name:str, outdir:Optional[str]=None,
-                  outnames:Union[str, List[str]]="{basename}.json", 
-                  simplified_output:bool=False, parallel:int=-1):
-        self.sanity_check()
-        if outdir is None:
-            save_as = repeat(None)
+        if n_bins_pdf is None:
+            n_bins_pdf = n_bins_data
+        binning_data = ROOT.RooFit.Binning(n_bins_data, bin_range[0], bin_range[1])
+        binning_pdf = ROOT.RooFit.Binning(n_bins_pdf, bin_range[0], bin_range[1])
+        from quickstats.interface.root import RooAbsData
+        h_data = RooAbsData.create_histogram(data, uuid.uuid4().hex, observable, binning_data)
+        h_pdf = RooAbsData.create_histogram(pdf, uuid.uuid4().hex, observable, binning_pdf)
+        h_pdf_data_binning = RooAbsData.create_histogram(pdf, uuid.uuid4().hex, observable,
+                                                         binning_data)
+        from quickstats.interface.root import TH1
+        if bin_error:
+            bin_errors = TH1.GetPoissonError(h_data.bin_content)
         else:
-            if isinstance(outnames, list):
-                save_as = [os.path.join(outdir, outname) for outname in outnames]
-            else:
-                save_as = []
-                for fname in fnames:
-                    basename = os.path.splitext(os.path.basename(fname))[0]
-                    save_as.append(os.path.join(oudir, outnames.format(basename=basename)))
+            size = len(h_data.bin_content)
+            bin_errors = {"lo": np.zeros(size), "hi": np.zeros(size)}
+        norm_data  = h_data.bin_content.sum()
+        norm_pdf1  = h_pdf.bin_content.sum()
+        norm_pdf2  = h_pdf_data_binning.bin_content.sum()
+        hist_data = {
+            'data': {
+                'x': h_data.bin_center,
+                'y': h_data.bin_content,
+                'yerrlo': bin_errors['lo'],
+                'yerrhi': bin_errors['hi']
+            },
+            'pdf': {
+                'x': h_pdf.bin_center,
+                'y': h_pdf.bin_content * norm_data / norm_pdf1 * (n_bins_pdf / n_bins_data)
+            },
+            'pdf_data_binning': {
+                'x': h_pdf_data_binning.bin_center,
+                'y': h_pdf_data_binning.bin_content * norm_data / norm_pdf2
+            }
+        }
+        return hist_data
+        
+
+    def run(self, fname:str, tree_name:str, save_summary_as:Optional[str]=None, 
+            save_param_as:Optional[str]=None, save_plot_as:Optional[str]=None):
+        export_hist_data = save_plot_as is not None
+        summary = self._run(fname, tree_name, model_class=self.model_class, 
+                            param_templates=self.param_templates,
+                            fit_options=self.fit_options,
+                            plot_options=self.plot_options,
+                            save_summary_as=save_summary_as,
+                            save_param_as=save_param_as,
+                            save_plot_as=save_plot_as)
+        return summary
+    
+    @semistaticmethod
+    def create_plot(self, data, outname:str, xlabel:Optional[str]=None,
+                    ylabel:Optional[str]=None, param_data:Optional[Dict]=None,
+                    stat_data:Optional[Dict]=None, plot_comparison:bool=True,
+                    comparison_mode:str="difference", styles:Optional[Dict]=None,
+                    annotations:Optional[List]=None):
+        from quickstats.plots import PdfDistributionPlot
+        label_map = {
+            "pdf": "fit"
+        }
+        plotter = PdfDistributionPlot(data, label_map=label_map, styles=styles)
+        annotation = {}
+        if param_data is not None:
+            annotation["params"] = param_data
+        if stat_data is not None:
+            annotation["stats"] = stat_data
+        if annotation:
+            plotter.set_annotation(**annotation)
+        if plot_comparison:
+            comparison_options = {
+                "reference": "data",
+                "target": "pdf_data_binning",
+                "mode": comparison_mode
+            }
+        else:
+            comparison_options = None
+        if (ylabel is not None) and "bin_width" in ylabel:
+            bin_width = np.unique(np.diff(data['data']['x']))
+            if (len(bin_width) > 1) and (not np.allclose(bin_width, bin_width[0])):
+                raise RuntimeError("can not deduce bin width: non-uniform binnings detected")
+            bin_width = bin_width[0]
+            ylabel = ylabel.format(bin_width=bin_width)
+            
+        if isinstance(annotations, dict):
+            plotter.add_annotation(**annotations)
+        elif isinstance(annotations, list):
+            for annotation in annotations:
+                plotter.add_annotation(**annotation)
+        ax = plotter.draw(xlabel=xlabel, ylabel=ylabel, targets=["data", "pdf"],
+                          comparison_options=comparison_options)
+        import matplotlib.pyplot as plt
+        plt.savefig(outname, bbox_inches="tight")
+    
+    @staticmethod
+    def _get_iter_save_paths(fnames:List[str], save_paths:Optional[Union[List[str], str]]=None):
+        # do not save
+        if save_paths is None:
+            return repeat(None)
+        # save by expressionn
+        elif isinstance(save_paths, str):
+            _save_paths = []
+            for fname in fnames:
+                basename = os.path.basename(fname)
+                _save_paths.append(fname.format(basename=basename))
+            return _save_paths
+        # save by explicit paths
+        elif isinstance(save_paths, list):
+            if len(fnames) != len(save_paths):
+                raise RuntimeError("number of output file paths must match the number input files")
+            return save_paths
+        else:
+            raise ValueError("invalid save option")
+        
+    def batch_run(self, fnames:List[str], tree_name:str,
+                  save_summary_as:Optional[Union[List[str], str]]=None,
+                  save_param_as:Optional[Union[List[str], str]]=None,
+                  save_plot_as:Optional[Union[List[str], str]]=None,
+                  plot_options:Optional[List]=None, parallel:int=-1):
+        self.sanity_check()
+        save_summary_as = self._get_iter_save_paths(fnames, save_summary_as)
+        save_param_as   = self._get_iter_save_paths(fnames, save_param_as)
+        save_plot_as    = self._get_iter_save_paths(fnames, save_plot_as)
+        if plot_options is None:
+            plot_options = repeat(self.plot_options)
+        elif isinstance(plot_options, dict):
+            plot_options = repeat(combine_dict(self.plot_options, plot_options))
+        elif isinstance(plot_options, list):
+            plot_options = [combine_dict(self.plot_options, _plot_options) for _plot_options in plot_options]
+        else:
+            raise ValueError("invalid plot options")
         args = (fnames, repeat(tree_name), repeat(self.model_class),
                 repeat(self.param_templates), repeat(self.fit_options),
-                save_as, repeat(simplified_output))
+                plot_options, save_summary_as, save_param_as, 
+                save_plot_as)
         results = execute_multi_tasks(self._run, *args, parallel=parallel)
         return results
     
     def run_over_categories(self, prefix:str, categories:List[str],
                             tree_name:str, input_dir:str="./",
-                            simplified_output:bool=True,
-                            outdir:Optional[str]=None,
-                            save_name:str="model_parameters.json",
+                            save_summary_as:Optional[Union[List[str], str]]=None,
+                            save_param_as:Optional[Union[List[str], str]]=None,
+                            save_plot_as:Optional[Union[List[str], str]]=None,
+                            save_merged_param_as:str="model_parameters.json",
+                            plot_options:Optional[List]=None,
                             parallel:int=-1):
         fnames = []
         for category in categories:
             fname = os.path.join(input_dir, f"{prefix}_{category}.root")
             fnames.append(fname)
 
-        if outdir is None:
-            save_as = repeat(None)
-        else:
-            save_as = []
-            basename = os.path.splitext(os.path.basename(save_name))[0]
-            for category in categories:
-                save_as.append(f"{basename}.json")
-        args = (fnames, repeat(tree_name), repeat(self.model_class),
-                repeat(self.param_templates), repeat(self.fit_options),
-                save_as, repeat(simplified_output))
-        results = execute_multi_tasks(self._run, *args, parallel=parallel)
-        summary = {}
-        for category, result in zip(categories, results):
-            summary[category] = result
-        if outdir is not None:
-            outname = os.path.join(outdir, save_name)
-            json.dump(summary, open(outname, "w"), indent=2)
-        return summary
\ No newline at end of file
+        results = self.batch_run(fnames, tree_name,
+                                 save_summary_as=save_summary_as,
+                                 save_param_as=save_param_as,
+                                 save_plot_as=save_plot_as,
+                                 plot_options=plot_options,
+                                 parallel=parallel)
+        category_results = dict(zip(categories, results))
+        if save_merged_param_as is not None:
+            param_data = {}
+            for category, result in category_results.items():
+                param_data[category] = self._get_simplified_param_data(result['parameters'])
+            with open(save_merged_param_as, "w") as outfile:
+                json.dump(param_data, outfile, indent=2)
+        return category_results
\ No newline at end of file
