@@ -97,8 +97,11 @@ class XMLWSBuilder(XMLWSBase):
                  use_binned:bool=False,
                  data_storage_type:str="vector",
                  minimizer_config:Optional[Dict]=None,
+                 unlimited_stack:bool=True,
                  verbosity:Optional[Union[int, str]]="INFO"):
-        super().__init__(source=source, basedir=basedir, verbosity=verbosity)
+        super().__init__(source=source, basedir=basedir,
+                         unlimited_stack=unlimited_stack,
+                         verbosity=verbosity)
         self.use_binned = use_binned
         self.data_storage_type = data_storage_type
         self.minimizer_config = minimizer_config
@@ -355,20 +358,17 @@ class XMLWSBuilder(XMLWSBase):
         return output_plot_path
                          
     def generate_asimov(self):
-        n_asimov = len(self.core_config['asimov_definitions'])
-        if n_asimov == 0:
-            return None
+        ws = self.comb_ws
+        asimov_definitions = self.core_config['asimov_definitions']
         if self.use_binned:
             data_name = self.core_config['dataset_name'] + "binned"
             self.stdout.warning("REGTEST: Fitting binned dataset.", "red")
         else:
             data_name = self.core_config['dataset_name']
-        ws = self.comb_ws
         range_name = self.get_fit_range_name()
-        asimov_handler = AsimovHandler(ws, data_name=data_name, range_name=range_name,
-                                       minimizer_config=self.minimizer_config)
-        for attributes in self.core_config['asimov_definitions']:
-            asimov_handler.generate_single_asimov(attributes)
+        self._generate_asimov(ws, asimov_definitions=asimov_definitions,
+                              data_name=data_name, range_name=range_name,
+                              minimizer_config=self.minimizer_config)
         
     def get_fit_range_name(self):
         do_blind = self.core_config['do_blind']
