@@ -5,7 +5,6 @@
 import os
 import re
 import json
-import time
 from typing import Optional, Union, List, Dict
 
 try:
@@ -25,6 +24,7 @@ from quickstats.utils.root_utils import load_macro, get_macro_dir, is_corrupt
 from quickstats.utils.common_utils import format_delimiter_enclosed_text
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, BracketType
+from quickstats.decorators import Timer
 
 from quickstats.utils.enums import GeneralEnum
     
@@ -52,15 +52,14 @@ class XMLWSCombiner(XMLWSBase):
                  minimizer_config:Optional[Dict]=None,
                  unlimited_stack=True,
                  verbosity:Optional[Union[int, str]]="INFO") -> None:
-        t0 = time.time()
         super().__init__(source=source, basedir=basedir,
                          unlimited_stack=unlimited_stack,
                          verbosity=verbosity)
         self.minimizer_config = minimizer_config
         self.load_extension()
-        self.initialize(source=source)
-        t1 = time.time()
-        self.init_time = t1 - t0
+        with Timer() as t:
+            self.initialize(source=source)
+        self.init_time = t.interval
         
     def initialize(self, source:Union[str, Dict]) -> None:
         self.combination_config = {}
@@ -259,9 +258,8 @@ class XMLWSCombiner(XMLWSBase):
             "pdf"  : {},
             "var"  : {}
         }
+        self.channel_config[channel]['renamed_constraint'] = {}
         self.stdout.info(f"INFO: Checking XML input for the channel \"{channel}\".")
-        model = self._get_channel_model(channel_config)
-        self.channel_config[channel]["model"] = model
         # walk through child nodes
         child_nodes = node["children"]
         for child_node in child_nodes:
@@ -285,13 +283,77 @@ class XMLWSCombiner(XMLWSBase):
                 self.read_rename_syst_node(channel, child_node)
             else:
                 raise RuntimeError(f"unknown channel item `{tag}`")
-    
+                
+    def validate_channel_input(self, channel:str):
+        self._channel_check(channel)
+        self.stdout.info(f"INFO: Checking input workspace for the channel \"{channel}\".")
+        channel_config = self.channel_config[channel]
+        model = self._get_channel_model(channel_config)
+        strict_mode = self.combination_config["strict"]
+        ws_filename = model.fname
+        renamed_pdfs = list(channel_config["rename_map"]["pdf"])
+        renamed_vars = list(channel_config["rename_map"]["var"])
+        renamed_pois = list(channel_config["poi_map"].values())
+        renamed_constraint = channel_config["renamed_constraint"]
+        missing_obj_msg = f'{{obj_type}} "{{obj_name}}" does not exist in the workspace "{ws_filename}"'
+        for pdf_name in renamed_pdfs:
+            nuis_name = renamed_constraint[pdf_name]["nuis"]
+            glob_name = renamed_constraint[pdf_name]["glob"]
+            renamed_vars.remove(nuis_name)
+            renamed_vars.remove(glob_name)            
+            pdf = model.workspace.pdf(pdf_name)
+            if not pdf:
+                msg = missing_obj_msg.format(obj_type="Constraint PDF", obj_name=pdf_name)
+                self._exception_check(msg, strict_mode)
+                continue
+            if not isinstance(pdf, ROOT.RooGaussian):
+                msg = f"constraint PDF \"{pdf_name}\" that is not an instance of RooGaussian is not supported"
+                self._exception_check(msg, strict_mode)
+                continue
+            nuis = model.workspace.var(nuis_name)
+            if not nuis:
+                msg = missing_obj_msg.format(obj_type="Nuisance parameter", obj_name=nuis_name)
+                raise RuntimeError(msg)
+            glob = model.workspace.var(glob_name)
+            if not glob:
+                msg = missing_obj_msg.format(obj_type="Global observable", obj_name=glob_name)
+                raise RuntimeError(msg)
+            nuis_check, glob_check, sigma_check = False, False, False
+            # this part can be optimized after ROOT 6.26
+            server_iter = pdf.serverIterator()
+            while True:
+                server = server_iter.Next()
+                if not server:
+                    break
+                server_name = server.GetName()
+                if server_name == nuis_name:
+                    nuis_check = True
+                elif server_name == glob_name:
+                    glob_check = True
+                elif abs(server.getVal() - 1) < self.EPSILON:
+                    sigma_check = True
+            if not sigma_check:
+                raise RuntimeError(f"sigma of constraint pdf \"{pdf_name}\" in the workspace \"{ws_filename}\" is not unity")
+        for var_name in renamed_vars:
+            if not model.workspace.var(var_name):
+                msg = missing_obj_msg.format(obj_type="variable", obj_name=var_name)
+                self._exception_check(msg, strict_mode)
+                continue
+        for poi_name in renamed_pois:
+            if poi_name == self.KEYWORDS['dummy']:
+                continue
+            # check poi exist in workspace
+            if not model.workspace.var(poi_name):
+                raise RuntimeError(f"poi \"{poi_name}\" does not exist in the channel \"{channel}\"")
+        # fix unspecified names
+        channel_config['workspace_name'] = model.workspace.GetName()
+        channel_config['model_config_name'] = model.model_config.GetName()
+        channel_config['data_name'] = model.data.GetName()            
+        self.stdout.info(f"INFO: Successfully validated input workspace for the channel \"{channel}\".")
+        
     def read_rename_syst_node(self, channel:str, node:Dict) -> None:
         self._channel_check(channel)
         channel_config = self.channel_config[channel]
-        model = channel_config["model"]
-        strict_mode = self.combination_config["strict"]
-        ws_filename = model.fname
         old_name = self._get_node_attrib(node, "OldName", required=True)
         new_name = self._get_node_attrib(node, "NewName", required=True)
         if not old_name:
@@ -300,6 +362,7 @@ class XMLWSCombiner(XMLWSBase):
         if not new_name:
             raise ValueError(f"missing attribute \"NewName\" from the Syst node (channel = {channel}):\n"
                              f"{node}")
+        renamed_constraint = channel_config["renamed_constraint"]
         pdf_map  = channel_config["rename_map"]["pdf"]
         var_map  = channel_config["rename_map"]["var"]
         _, object_type = self._get_object_name_and_type_from_expr(new_name)
@@ -317,44 +380,6 @@ class XMLWSCombiner(XMLWSBase):
                 raise ValueError(f"invalid format for constraint pdf \"{old_name}\" in channel \"{channel}\"")
             nuis_name = components[0]
             glob_name = components[1]
-
-            pdf = model.workspace.pdf(pdf_name)
-            if not pdf:
-                msg = f"constraint PDF \"{pdf_name}\" does not exist in the workspace \"{ws_filename}\""
-                self._exception_check(msg, strict_mode)
-                return None
-            if not isinstance(pdf, ROOT.RooGaussian):
-                msg = f"constraint PDF \"{pdf_name}\" that is not an instance of RooGaussian is not supported"
-                self._exception_check(msg, strict_mode)
-                return None
-            nuis = model.workspace.var(nuis_name)
-            if not nuis:
-                raise RuntimeError(f"nuisance parameter \"{nuis_name}\" does not exist in the workspace \"{ws_filename}\"")
-            glob = model.workspace.var(glob_name)
-            if not glob:
-                raise RuntimeError(f"global observable \"{nuis_name}\" does not exist in the workspace \"{ws_filename}\"")
-            nuis_check, glob_check, sigma_check = False, False, False
-            # this part can be optimized after ROOT 6.26
-            server_iter = pdf.serverIterator()
-            while True:
-                server = server_iter.Next()
-                if not server:
-                    break
-                server_name = server.GetName()
-                if server_name == nuis_name:
-                    nuis_check = True
-                elif server_name == glob_name:
-                    glob_check = True
-                elif abs(server.getVal() - 1) < self.EPSILON:
-                    sigma_check = True
-            if not sigma_check:
-                raise RuntimeError(f"sigma of constraint pdf \"{pdf_name}\" in the workspace \"{ws_filename}\" is not unity")
-            if not nuis_check:
-                raise RuntimeError(f"nuisance parameter \"{nuis_name}\" is not a dependent of the constraint PDF "
-                                   f"\"{pdf_name}\" in the workspace \"{ws_filename}\"")
-            if not glob_check:
-                raise RuntimeError(f"global observable \"{glob_name}\" is not a dependent of the constraint PDF "
-                                   f"\"{pdf_name}\" in the workspace \"{ws_filename}\"")
             if nuis_name in var_map:
                 raise ValueError(f"Old nuisance parameter \"{new_name}\" is duplicated in the channel \"{channel}\"")
             var_map[nuis_name] = new_name
@@ -364,12 +389,8 @@ class XMLWSCombiner(XMLWSBase):
             if pdf_name in pdf_map:
                 raise ValueError(f"Old PDF \"{pdf_name}\" is duplicated in the channel \"{channel}\"")
             pdf_map[pdf_name] = new_name + self.KEYWORDS["constrterm_suffix"]
+            renamed_constraint[pdf_name] = {"nuis": nuis_name, "glob": glob_name}
         elif WSObjectType.DEFINED:
-            var = model.workspace.var(old_name)
-            if not var:
-                msg = f"variable \"{old_name}\" does not exist in the workspace {ws_filename}"
-                self._exception_check(msg, strict_mode)
-                return None
             if old_name in var_map:
                 raise ValueError(f"Old nuisance parameter \"{old_name}\" is duplicated in the channel \"{channel}\"")
             var_map[old_name] = new_name
@@ -377,7 +398,6 @@ class XMLWSCombiner(XMLWSBase):
     def read_channel_poi_node(self, channel:str, node:Dict) -> None:
         self._channel_check(channel)
         channel_config = self.channel_config[channel]
-        model = channel_config["model"]
         pois = self._get_node_attrib(node, "Input", required=True, dtype="str_list")
         if "poi_definition" not in self.combination_config:
             raise RuntimeError("Combined POIs not initialized")
@@ -386,12 +406,11 @@ class XMLWSCombiner(XMLWSBase):
             raise RuntimeError(f"channel \"{channel}\" has more POIs than the combined model")
         poi_check = {}
         for poi in pois:
+            if poi == self.KEYWORDS['dummy']:
+                continue
             # check for duplication
             if poi in poi_check:
                 raise RuntimeError(f"POI \"{poi}\" for the channel \"{channel}\" is defined more than once.")
-            # check poi exist in workspace
-            if not model.workspace.arg(poi):
-                raise RuntimeError(f"poi \"{poi}\" does not exist in the channel \"{channel}\"")
             poi_check[poi] = None
         dummy_pois = [self.KEYWORDS['dummy']] * (len(comb_pois) - len(pois))
         pois += dummy_pois
@@ -432,12 +451,11 @@ class XMLWSCombiner(XMLWSBase):
     def get_channel_summary(self, channel:str, indent:int=4) -> str:
         self._channel_check(channel)
         channel_config = self.channel_config[channel]
-        model = channel_config["model"]
         summary_text = ""
-        summary_text += " "*indent + f" Input File Name: {model.fname}" + "\n"
-        summary_text += " "*indent + f"  Workspace Name: {model.workspace.GetName()}" + "\n"
-        summary_text += " "*indent + f"ModelConfig Name: {model.model_config.GetName()}" + "\n"
-        summary_text += " "*indent + f"       Data Name: {model.data.GetName()}" + "\n"
+        summary_text += " "*indent + f" Input File Name: {channel_config['input_file']}" + "\n"
+        summary_text += " "*indent + f"  Workspace Name: {channel_config['workspace_name']}" + "\n"
+        summary_text += " "*indent + f"ModelConfig Name: {channel_config['model_config_name']}" + "\n"
+        summary_text += " "*indent + f"       Data Name: {channel_config['data_name']}" + "\n"
         return summary_text
 
     def get_poi_table(self) -> str:
@@ -537,7 +555,7 @@ class XMLWSCombiner(XMLWSBase):
             else:
                 ignore_list.append(comb_poi_name)
             if comb_poi_name == channel_poi_name:
-                self.stdout.debug(f'DEBUG: Found redundant renaming of the poi "{old_name}". Skipped.')
+                self.stdout.debug(f'DEBUG: Found redundant renaming of the poi "{channel_poi_name}". Skipped.')
                 continue
             poi.SetName(comb_poi_name)
             self.stdout.debug(f'DEBUG: Renamed poi from "{channel_poi_name}" to "{comb_poi_name}"')
@@ -798,14 +816,25 @@ class XMLWSCombiner(XMLWSBase):
             self.stdout.info(f'INFO: Saving final combined workspace to "{filename}"')
             self.comb_ws.writeToFile(filename, True)
     
-    def run(self, save_rename_ws:bool=False, save_combine_ws:bool=False,
-            save_final_ws:bool=True) -> None:
-        t0 = time.time()
-        self.print_input_summary()
-        self.rename(save_rename_ws=save_rename_ws)
-        self.combine(load_rename_ws=save_rename_ws, save_combine_ws=save_combine_ws)
-        self.finalize(load_combine_ws=save_combine_ws, save_final_ws=save_final_ws)
-        t1 = time.time()
-        comb_time  = t1 - t0
-        total_time = self.init_time + comb_time
-        self.stdout.info(f"INFO: Total time taken: {total_time:.3f}s")
\ No newline at end of file
+    def create_combined_workspace(self, infiles:Optional[Dict[str, str]]=None,
+                                  outfile:Optional[str]=None,
+                                  save_rename_ws:bool=False,
+                                  save_combine_ws:bool=False,
+                                  save_final_ws:bool=True) -> None:
+        with Timer() as t:
+            # override path to the input channel workspaces
+            if infiles is not None:
+                for channel in self.channel_config:
+                    if channel not in infiles:
+                        raise ValueError(f'missing input workspace path for the channel "{channel}"')
+                    self.channel_config[channel]['input_file'] = infiles[channel]
+            # override path to the combined workspace
+            if outfile is not None:
+                self.combination_config['output_file'] = outfile
+            for channel in self.channel_config:
+                self.validate_channel_input(channel)
+            self.print_input_summary()
+            self.rename(save_rename_ws=save_rename_ws)
+            self.combine(load_rename_ws=save_rename_ws, save_combine_ws=save_combine_ws)
+            self.finalize(load_combine_ws=save_combine_ws, save_final_ws=save_final_ws)
+        self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
\ No newline at end of file
