@@ -17,11 +17,11 @@ from quickstats import semistaticmethod
 from quickstats.components import AbstractObject, ExtendedModel
 from quickstats.utils.io import Verbosity
 from quickstats.utils.root_utils import load_macro, get_macro_dir
-from quickstats.utils.common_utils import format_delimiter_enclosed_text
+from quickstats.utils.common_utils import format_delimiter_enclosed_text, remove_list_duplicates
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.basics import WSArgument
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, AsimovHandler
-
+from quickstats.decorators import Timer
 from quickstats.utils.enums import GeneralEnum
 
 class ModifierAction(GeneralEnum):
@@ -49,7 +49,8 @@ class XMLWSModifier(XMLWSBase):
             'redefine': [],
             'asimov': [],
             'constraint': [],
-            'add_product_terms': {},            
+            'add_product_terms': {},
+            'add_syst': []
         }
         self.actions['rename'] = {
             'workspace'   : {},
@@ -101,7 +102,8 @@ class XMLWSModifier(XMLWSBase):
             "redefine"   : _actions.pop("redefine", []),
             "asimov"     : _actions.pop("asimov", []),
             "constraint" : [],
-            "add_product_terms": _actions.pop("add_product_terms", {})
+            "add_product_terms": _actions.pop("add_product_terms", {}),
+            'add_syst': _actions.pop("add_syst", [])
         }
         
         # handle constraint definition
@@ -203,6 +205,9 @@ class XMLWSModifier(XMLWSBase):
                 raise ValueError("AddProductTerm action defined multiple times for the variable "
                                  f"\"{name}\"")
             self.actions['add_product_terms'][name] = extra_terms
+        elif tag == "AddNPSet":
+            nuis_names = self._get_node_attrib(node, "Name", required=True, dtype="str_list")
+            self.actions['add_syst'] += nuis_names
         else:
             raise RuntimeError(f"unknown item `{tag}`")
     
@@ -249,82 +254,88 @@ class XMLWSModifier(XMLWSBase):
         if len(self.actions['rename']['model_config']) > 1:
             raise RuntimeError("model config is renamed more than once")
     
-    def create_modified_workspace(self):
-        self.sanity_check()
-        filename = self.config['input_file']
-        ws_name  = self.config['workspace_name']
-        mc_name  = self.config['model_config_name']
-        self.stdout.info(f"INFO: Begin modification of the workspace \"{filename}\".")
-        model    = ExtendedModel(filename, ws_name=ws_name, mc_name=mc_name,
-                                 data_name=None, verbosity="WARNING")
-        ws_orig  = model.workspace
-        mc_orig  = model.model_config
-        ws_name  = ws_orig.GetName()
-        mc_name  = mc_orig.GetName()
-        
-        # create temporary workspace
-        ws_tmp = ROOT.RooWorkspace(ws_name)
-    
-        title_str = format_delimiter_enclosed_text("Step 1: Redefine objects", "-")
-        self.stdout.info(title_str)
-        flag = self.redefine_objects(ws_orig, ws_tmp)
-        # nothing is done
-        if not flag:
-            self.stdout.info("INFO: No objects are redefined.")
-        
-        title_str = format_delimiter_enclosed_text("Step 2: Create new objects", "-")
-        self.stdout.info(title_str)
-        flag = self.implement_external_pdfs(ws_tmp)
-        flag |= self.implement_objects(ws_tmp)
-        # nothing is done
-        if not flag:
-            self.stdout.info("INFO: No objects are defined.")
-            
-        # need to import datasets first so the variables are also imported into the workspace
-        self.import_datasets(ws_orig, ws_tmp)
-        
-        title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
-        self.stdout.info(title_str)
-        rename_map = self.get_rename_map(ws_orig, ws_tmp)
-        actual_rename_map = self.get_actual_rename_map(rename_map, ws_orig, ws_tmp)
-        self.rename_variables(mc_orig, ws_tmp, rename_map)
-        if not rename_map:
-            self.stdout.info("INFO: No variables are renamed.")
-        
-        title_str = format_delimiter_enclosed_text("Step 4: Making output workspace", "-")
-        self.stdout.info(title_str)
+    def create_modified_workspace(self, infile:Optional[str]=None, outfile:Optional[str]=None):
+        with Timer() as t:
+            self.sanity_check()
+            # override path to the input workspace
+            if infile is None:
+                infile = self.config['input_file']
+            # override path to the output workspace
+            if outfile is None:
+                outfile = self.config['output_file']
+            ws_name  = self.config['workspace_name']
+            mc_name  = self.config['model_config_name']
+            self.stdout.info(f"INFO: Begin modification of the workspace \"{infile}\".")
+            model    = ExtendedModel(infile, ws_name=ws_name, mc_name=mc_name,
+                                     data_name=None, verbosity="WARNING")
+            ws_orig  = model.workspace
+            mc_orig  = model.model_config
+            ws_name  = ws_orig.GetName()
+            mc_name  = mc_orig.GetName()
 
-        # create final workspace
-        if len(self.actions['constraint']) > 0:
-            ws_final = ROOT.RooWorkspace(ws_name)
-            sim_pdf = self.remake_simultaneous_pdf(mc_orig, ws_tmp)
-            self.import_object(ws_final, sim_pdf)
-            self.import_datasets(ws_orig, ws_final)
-        else:
-            ws_final = ws_tmp        
-            
-        # post processing pdfs, functions and variables
-        self.post_process_workspace(ws_final)
+            # create temporary workspace
+            ws_tmp = ROOT.RooWorkspace(ws_name)
 
-        mc_final = self.create_model_config(mc_orig, ws_final, actual_rename_map)
-        
-        self.rename_objects(ws_final, mc_final)
-        
-        self.import_object(ws_final, mc_final, silent=False)
-        self.import_class_code(ws_final)
-        
-        self.create_snapshots(ws_orig, mc_orig, ws_final, mc_final, rename_map)
-        
-        self.generate_asimov(ws_final)
-        
-        self.setup_parameters(ws_final)
+            title_str = format_delimiter_enclosed_text("Step 1: Redefine objects", "-")
+            self.stdout.info(title_str)
+            flag = self.redefine_objects(ws_orig, ws_tmp)
+            # nothing is done
+            if not flag:
+                self.stdout.info("INFO: No objects are redefined.")
 
-        outname = self.config['output_file']
-        outdir  = os.path.dirname(outname)
-        if outdir and (not os.path.exists(outdir)):
-            os.makedirs(outdir)
-        ws_final.writeToFile(outname, True)
-        self.stdout.info(f"INFO: Saved output workspace as \"{outname}\".")
+            title_str = format_delimiter_enclosed_text("Step 2: Create new objects", "-")
+            self.stdout.info(title_str)
+            flag = self.implement_external_pdfs(ws_tmp)
+            flag |= self.implement_objects(ws_tmp)
+            # nothing is done
+            if not flag:
+                self.stdout.info("INFO: No objects are defined.")
+
+            # need to import datasets first so the variables are also imported into the workspace
+            self.import_datasets(ws_orig, ws_tmp)
+
+            title_str = format_delimiter_enclosed_text("Step 3: Rename variables", "-")
+            self.stdout.info(title_str)
+            rename_map = self.get_rename_map(ws_orig, ws_tmp)
+            actual_rename_map = self.get_actual_rename_map(rename_map, ws_orig, ws_tmp)
+            self.rename_variables(mc_orig, ws_tmp, rename_map)
+            if not rename_map:
+                self.stdout.info("INFO: No variables are renamed.")
+
+            title_str = format_delimiter_enclosed_text("Step 4: Making output workspace", "-")
+            self.stdout.info(title_str)
+
+            # create final workspace
+            if len(self.actions['constraint']) > 0:
+                ws_final = ROOT.RooWorkspace(ws_name)
+                sim_pdf = self.remake_simultaneous_pdf(mc_orig, ws_tmp)
+                self.import_object(ws_final, sim_pdf)
+                self.import_datasets(ws_orig, ws_final)
+            else:
+                ws_final = ws_tmp        
+
+            # post processing pdfs, functions and variables
+            self.post_process_workspace(ws_final)
+
+            mc_final = self.create_model_config(mc_orig, ws_final, actual_rename_map)
+
+            self.rename_objects(ws_final, mc_final)
+
+            self.import_object(ws_final, mc_final, silent=False)
+            self.import_class_code(ws_final)
+
+            self.create_snapshots(ws_orig, mc_orig, ws_final, mc_final, rename_map)
+
+            self.generate_asimov(ws_final)
+
+            self.setup_parameters(ws_final)
+
+            outdir  = os.path.dirname(outfile)
+            if outdir and (not os.path.exists(outdir)):
+                os.makedirs(outdir)
+            ws_final.writeToFile(outfile, True)
+            self.stdout.info(f"INFO: Saved output workspace as \"{outfile}\".")
+        self.stdout.info(f"INFO: Total time taken: {t.interval:.3f}s")
     
     def post_process_workspace(self, ws:ROOT.RooWorkspace):
         self.add_product_terms(ws)
@@ -500,12 +511,28 @@ class XMLWSModifier(XMLWSBase):
         else:
             glob_names = []
             
-        # additional nuisance_parameters and global observables
+        # newly defined nuisance_parameters and global observables
         additional_constraints = self.actions['constraint']
+        new_nuis = []
         for item in additional_constraints:
+            new_nuis   += item['nuis']
             nuis_names += item['nuis']
             glob_names += item['glob']
+            
+        # additional (unconstrained) nuisance parameters
+        extra_nuis = self.actions['add_syst']
+        nuis_names += extra_nuis
         
+        if len(new_nuis) > 0:
+            self.stdout.info("INFO: The following newly defined nuisance parameter(s) will be added to ModelConfig")
+            for nuis_name in new_nuis:
+                self.stdout.info(f"    {nuis_name}")
+        if len(extra_nuis) > 0:
+            self.stdout.info("INFO: The following additional nuisance parameter(s) will be added to ModelConfig")
+            for nuis_name in extra_nuis:
+                self.stdout.info(f"    {nuis_name}")
+        
+        nuis_names = remove_list_duplicates(nuis_names)
         # set nuisance parameters
         nuisance_parameters = ROOT.RooArgSet()
         for nuis_name in nuis_names:
