@@ -216,6 +216,20 @@ ROOT_MACROS = \
                  target->add(RooArgSet(*target_var, *weight), weight_val);
              }
         }
+        
+        struct RooArgSetInfo {
+          std::vector<std::string> class_names;
+          std::vector<std::string> names;
+          RooArgSetInfo(const RooArgSet* argset){
+            const size_t n_obj = argset->size();
+            this->class_names = std::vector<std::string>(n_obj);
+            this->names = std::vector<std::string>(n_obj);
+            for (size_t i = 0; i < n_obj; i++){
+                this->class_names[i] = ((TObject*)(*argset)[i])->ClassName();
+                this->class_names[i] = ((TObject*)(*argset)[i])->GetName();
+            }
+          }
+        } ;
     };
     """
 }
