@@ -100,4 +100,20 @@ def str_decode_value(val_str):
     return val
 
 def is_nan_or_inf(value):
-    return np.isnan(value) or np.isinf(value)
\ No newline at end of file
+    return np.isnan(value) or np.isinf(value)
+
+def get_bins_given_edges(low_edge:float, high_edge:float, nbins:int):
+    bin_width = (high_edge - low_edge) / nbins
+    low_bin_center  = low_edge + bin_width / 2
+    high_bin_center = high_edge - bin_width /2
+    bins = np.linspace(low_bin_center, high_bin_center, nbins)
+    return bins
+
+def array_issubset(a:np.ndarray, b:np.ndarray):
+    """
+    Check if array b is a subset of array a
+    """
+    a = np.unique(a)
+    b = np.unique(b)
+    c = np.intersect1d(a, b)
+    return c.size == b.size
\ No newline at end of file
