@@ -200,7 +200,7 @@ class ParamParser:
         # convert to numpy arrays
         combinations = [np.array(param_values[param_name]) for param_name in param_names]
         # take care of rounding errors and negative zeros
-        combinations = [(arr.round(decimals=8) + 0.0) for arr in combinations]
+        combinations = [(arr.round(decimals=8) + 0.0) if arr[0] is not None else arr for arr in combinations]
         # take care of duplicate ploints
         combinations = [np.unique(arr) for arr in combinations]
         combinations = itertools.product(*combinations)
