@@ -1,4 +1,4 @@
-from typing import Optional, Union, Dict, List
+from typing import Optional, Union, Dict, List, Tuple, Callable
 
 from cycler import cycler
 
@@ -20,6 +20,16 @@ class PdfDistributionPlot(AbstractPlot):
             "elinewidth": 1,
             "capsize": 3,
             "capthick": 1
+        },
+        "annotation": {
+            "xy": (0.05, 0.9),
+            "xycoords": "axes fraction",
+            "horizontalalignment": "left",
+            "verticalalignment": "top",
+            "fontsize": 20
+        },
+        "legend": {
+            "borderpad": 1
         }
     }
     
@@ -55,6 +65,8 @@ class PdfDistributionPlot(AbstractPlot):
             self.label_map = label_map
 
         self.colors = {}
+        
+        self.annotation = None
 
     def get_error_data(self, data):
         xerr = data.get('xerr', None)
@@ -168,7 +180,8 @@ class PdfDistributionPlot(AbstractPlot):
             processed_data['sideband'] = ~blind
         return processed_data
     
-    def draw_comparison(self, ax, reference_data, target_data, mode:str="ratio",
+    def draw_comparison(self, ax, reference_data, target_data, 
+                        xlabel:str="", ylabel:str="",  mode:str="ratio",
                         plot_options:Optional[Dict]=None, show_errorbar:bool=True, 
                         blind_range:Optional[List[float]]=None,
                         draw_ratio_line:bool=True):
@@ -197,7 +210,7 @@ class PdfDistributionPlot(AbstractPlot):
                 else:
                     pdata_comp['yerr'] = pdata_ref['yerr'] / pdata_tgt['y']
         elif vmode == 1:
-            pdata_comp['y'] = pdata_ref['y'] / pdata_tgt['y']
+            pdata_comp['y'] = pdata_ref['y'] - pdata_tgt['y']
         combined_options = combine_dict(self.styles['errorbar'], plot_options)
         if combined_options.get('markersize', None) is None:
             nbins = len(pdata_comp['x'])
@@ -225,10 +238,44 @@ class PdfDistributionPlot(AbstractPlot):
             if draw_ratio_line:
                 ratio_linestyles = self.config['ratio_line_styles']
                 ax.axhline(1, zorder=0, **ratio_linestyles)
+        elif vmode == 1:
+            centralize_axis(ax, which="y", ref_value=0)
+            if draw_ratio_line:
+                ratio_linestyles = self.config['ratio_line_styles']
+                ax.axhline(0, zorder=0, **ratio_linestyles)
+        if not ylabel:
+            if vmode == 0:
+                ylabel = "Data / Fit"
+            elif vmode == 1:
+                ylabel = "Data - Fit"
+            else:
+                ylabel = ""
+        self.draw_axis_components(ax, xlabel=xlabel, ylabel=ylabel)
         return handle
-        
     
+    def set_annotation(self, params:Optional[Dict]=None,
+                       stats:Optional[Dict]=None,
+                       val_fmt:str="{:.3f}",
+                       **styles):
+        text = ""
+        if params:
+            for param, value in params.items():
+                if isinstance(value, float):
+                    value = val_fmt.format(value)
+                text += f"{param} = {value}\n"
+            text += "\n"
+        if stats:
+            for stat, value in stats.items():
+                if isinstance(value, float):
+                    value = val_fmt.format(value)
+                text += f"{stat} = {value}\n"
+        self.annotation = {
+            "text": text,
+            **styles
+        }
+
     def draw(self, xlabel:str="", ylabel:str="",
+             targets:Optional[List]=None,
              logx:bool=False, logy:bool=False,
              blind_range:Optional[List[float]]=None,
              show_errorbar:bool=True, 
@@ -238,8 +285,10 @@ class PdfDistributionPlot(AbstractPlot):
             ax, ax_ratio = self.draw_frame(ratio_frames, logx=logx, logy=logy)
         else:
             ax = self.draw_frame(logx=logx, logy=logy)
-        
+
         for name in self.collective_data:
+            if (targets is not None) and(name not in targets):
+                continue
             data = self.collective_data[name]
             label = self.label_map.get(name, name)
             plot_options = self.plot_options.get(name, {})
@@ -268,19 +317,24 @@ class PdfDistributionPlot(AbstractPlot):
             reference = comparison_options['reference']
             target = comparison_options['target']
             mode = comparison_options.get('mode', "ratio")
-            label = comparison_options.get('label', "Data / Fit")
+            ylabel = comparison_options.get('label', "")
             draw_ratio_line = comparison_options.get('draw_ratio_line', True)
             plot_options = self.plot_options.get(reference, {})
             plot_options['color'] = self.colors[reference]
             self.draw_comparison(ax_ratio,
                                  self.collective_data[reference],
                                  self.collective_data[target],
-                                 mode=mode,
+                                 xlabel=ax.get_xlabel(),
+                                 ylabel=ylabel, mode=mode,
                                  plot_options=plot_options,
                                  show_errorbar=show_errorbar,
                                  blind_range=blind_range,
                                  draw_ratio_line=draw_ratio_line)
-            self.draw_axis_components(ax_ratio, xlabel=ax.get_xlabel(), ylabel=label)
             ax.set(xlabel=None)
             ax.tick_params(axis="x", labelbottom=False)
+
+        if self.annotation:
+            kwargs = combine_dict(self.styles['annotation'], self.annotation)
+            ax.annotate(**kwargs)
+            
         return ax
\ No newline at end of file
