@@ -288,7 +288,7 @@ def set_axis_tick_styles(ax, styles=None):
         ax.set_major_locator(new_locator)
                 
 
-def centralize_axis(ax, which='x', ref_value:float=0):
+def centralize_axis(ax, which='x', ref_value:float=0, padding:float=0.1):
     if which == 'x':
         get_lim = ax.get_xlim
         set_lim = ax.set_xlim
@@ -297,7 +297,8 @@ def centralize_axis(ax, which='x', ref_value:float=0):
         set_lim = ax.set_ylim    
     limits = get_lim()
     vmax = np.max(np.abs([ref_value - limits[0], limits[1] - ref_value]))
-    set_lim(ref_value - vmax, ref_value + vmax)
+    pad = vmax * padding
+    set_lim(ref_value - vmax - pad, ref_value + vmax + pad)
         
 def parse_transform(obj:str):
     if not obj:
