@@ -3,14 +3,20 @@ import math
 
 import numpy as np
 
-import ROOT
-
 from quickstats.plots import AbstractPlot
 from quickstats.plots.template import single_frame, parse_styles
-from quickstats.utils.roostats_utils import get_hypotest_data
 
 class TestStatisticDistributionPlot(AbstractPlot):
     
+    STYLES = {
+        'legend': {
+            'fontsize': 12
+        },
+        'annotation':{
+            'fontsize': 12
+        }
+    }
+    
     COLOR_PALLETE = {
         'null': 'b',
         'alt': 'r',
@@ -18,8 +24,8 @@ class TestStatisticDistributionPlot(AbstractPlot):
         'alt_sec': 'tab:pink'
     }
         
-    def __init__(self, result:ROOT.RooStats.HypoTestResult,
-                 sec_result:Optional[ROOT.RooStats.HypoTestResult]=None,
+    def __init__(self, result:"ROOT.RooStats.HypoTestResult",
+                 sec_result:Optional["ROOT.RooStats.HypoTestResult"]=None,
                  teststat_str:str=r"\~{q}_{\mu}",
                  sec_teststat_str:str=r"\~{q}_{\mu}",
                  annotate_str:str=r"toys",
@@ -31,6 +37,7 @@ class TestStatisticDistributionPlot(AbstractPlot):
                          analysis_label_options=analysis_label_options)
         
         self.result = result
+        from quickstats.utils.roostats_utils import get_hypotest_data
         self.data = get_hypotest_data(self.result)
         self.teststat_str = teststat_str
         self.annotate_str = annotate_str
