@@ -32,6 +32,7 @@ class UpperLimit1DPlot(AbstractPlot):
         '2sigma': 'hh:darkyellow',
         '1sigma': 'hh:lightturquoise',
         'expected': 'k',
+        'third': 'k',
         'observed': 'k',
     }
     
@@ -39,6 +40,7 @@ class UpperLimit1DPlot(AbstractPlot):
         '2sigma': r'Expected limit $\pm 2\sigma$',
         '1sigma': r'Expected limit $\pm 1\sigma$',
         'expected': r'Expected limit',
+        'third': 'Third limit',
         'observed': r'Observed limit',
     }
 
@@ -49,6 +51,7 @@ class UpperLimit1DPlot(AbstractPlot):
     def __init__(self, category_df, label_map, line_below=None,
                  color_pallete:Optional[Dict]=None,
                  labels:Optional[Dict]=None,
+                 config:Optional[Dict]=None,
                  styles:Optional[Union[Dict, str]]=None,
                  analysis_label_options:Optional[Union[Dict, str]]=None):
         super().__init__(color_pallete=color_pallete,
@@ -59,10 +62,10 @@ class UpperLimit1DPlot(AbstractPlot):
         self.line_below = line_below
         
         self.labels = combine_dict(self.LABELS, labels)  
-        self.config = combine_dict(self.CONFIG)
+        self.config = combine_dict(self.CONFIG, config)
     
     def draw(self, logx:bool=False, xlabel:Optional[str]=None, markersize:float=50.,
-             draw_observed:bool=True, draw_stat:bool=False, sig_fig:int=2):
+             draw_observed:bool=True, draw_stat:bool=False, draw_third_column:Optional[str]=None, sig_fig:int=2):
         if (draw_observed + draw_stat) > 1:
             raise RuntimeError("draw_observed and draw_stat can not be both True")
         n_category = len(self.category_df.columns)
@@ -75,6 +78,8 @@ class UpperLimit1DPlot(AbstractPlot):
             text_pos = {'expected': 0.775, 'stat': 0.925}
         if (not draw_observed) and (not draw_stat):
             text_pos = {'expected': 0.925}
+        if draw_third_column:
+            text_pos = {'observed': 0.725, 'expected': 0.825, 'third': 0.925}
         
         for i, category in enumerate(self.category_df):
             df = self.category_df[category]
@@ -107,7 +112,17 @@ class UpperLimit1DPlot(AbstractPlot):
                     verticalalignment='center',
                     transform=transform,
                     **self.styles['text'])
-            # draw +1
+            # draw third
+            if draw_third_column:
+                third_limit = df['third']
+                ax.vlines(third_limit, i, i + 1, colors=self.color_pallete['third'], linestyles = 'dashed',
+                          zorder = 1.1, label=self.labels['third'] if i==0 else '')
+                ax.text(text_pos['third'], i + 0.5, f"{{:.{sig_fig}f}}".format(third_limit), 
+                        horizontalalignment='center', 
+                        verticalalignment='center',
+                        transform=transform,
+                        **self.styles['text'])
+            # draw error band
             ax.fill_betweenx([i, i + 1], df['-2'], df['2'], facecolor=self.color_pallete['2sigma'], 
                              label=self.labels['2sigma'] if i==0 else '')
             ax.fill_betweenx([i, i + 1], df['-1'], df['1'], facecolor=self.color_pallete['1sigma'], 
@@ -141,6 +156,12 @@ class UpperLimit1DPlot(AbstractPlot):
                     verticalalignment='center',
                     transform=transform,
                     **self.styles['text'])
+        if draw_third_column:
+            ax.text(text_pos['third'], n_category + 0.3, draw_third_column, 
+                    horizontalalignment='center', 
+                    verticalalignment='center',
+                    transform=transform,
+                    **self.styles['text'])
         ax.text(text_pos['expected'], n_category + 0.3, 'Exp.', 
                 horizontalalignment='center', 
                 verticalalignment='center',
@@ -152,11 +173,14 @@ class UpperLimit1DPlot(AbstractPlot):
         border_leg = patches.Rectangle((0, 0), 1, 1, facecolor = 'none', edgecolor = 'black', linewidth = 1)
 
         handles, labels = ax.get_legend_handles_labels()
-        if draw_observed:
+        if draw_observed and not draw_third_column:
             handles = [handles[0], handles[1], (handles[3], border_leg), (handles[2], border_leg)]
             labels  = [labels[0], labels[1], labels[3], labels[2]]
-        if draw_stat:
+        if draw_stat and not draw_third_column:
             handles = [handles[0], (handles[2], border_leg), (handles[1], border_leg)]
             labels  = [labels[0], labels[2], labels[1]]
+        if draw_third_column:
+            handles = [handles[0], handles[1], handles[2], (handles[4], border_leg), (handles[3], border_leg)]
+            labels  = [labels[0], labels[1], labels[2], labels[4], labels[3]]
         ax.legend(handles, labels, **self.styles['legend'])
         return ax
